"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.signTransaction = signTransaction;
const viem_1 = require("viem");
const actions_1 = require("viem/actions");
const chains_1 = require("viem/chains");
const utils_1 = require("viem/utils");
const constants_js_1 = require("../constants.js");
const eip712_js_1 = require("../eip712.js");
const account_js_1 = require("../errors/account.js");
const utils_js_1 = require("../utils.js");
const ALLOWED_CHAINS = [chains_1.abstractTestnet.id];
async function signTransaction(client, signerClient, args, useSignerAddress = false) {
    const { account: account_ = client.account, chain = client.chain, ...transaction } = args;
    transaction.type = 'eip712';
    (0, utils_js_1.transformHexValues)(transaction, [
        'value',
        'nonce',
        'maxPriorityFeePerGas',
        'gas',
        'value',
        'chainId',
        'gasPerPubdata',
    ]);
    if (!account_)
        throw new account_js_1.AccountNotFoundError({
            docsPath: '/docs/actions/wallet/signTransaction',
        });
    const smartAccount = (0, utils_1.parseAccount)(account_);
    const fromAccount = useSignerAddress ? signerClient.account : smartAccount;
    (0, eip712_js_1.assertEip712Request)({
        account: smartAccount,
        chain,
        ...transaction,
    });
    if (!chain || !ALLOWED_CHAINS.includes(chain.id)) {
        throw new viem_1.BaseError('Invalid chain specified');
    }
    if (!chain?.custom?.getEip712Domain)
        throw new viem_1.BaseError('`getEip712Domain` not found on chain.');
    if (!chain?.serializers?.transaction)
        throw new viem_1.BaseError('transaction serializer not found on chain.');
    const chainId = await (0, utils_1.getAction)(client, actions_1.getChainId, 'getChainId')({});
    if (chain !== null)
        (0, utils_1.assertCurrentChain)({
            currentChainId: chainId,
            chain: chain,
        });
    const eip712Domain = chain?.custom.getEip712Domain({
        ...transaction,
        chainId,
        from: fromAccount.address,
        type: 'eip712',
    });
    const rawSignature = await (0, actions_1.signTypedData)(signerClient, {
        ...eip712Domain,
        account: signerClient.account,
    });
    let signature;
    if (useSignerAddress) {
        signature = rawSignature;
    }
    else {
        signature = (0, viem_1.encodeAbiParameters)((0, viem_1.parseAbiParameters)(['bytes', 'address', 'bytes[]']), [rawSignature, constants_js_1.VALIDATOR_ADDRESS, []]);
    }
    return chain?.serializers?.transaction({
        chainId,
        ...transaction,
        from: fromAccount.address,
        customSignature: signature,
        type: 'eip712',
    }, { r: '0x0', s: '0x0', v: 0n });
}
//# sourceMappingURL=signTransaction.js.map