import { BaseError, encodeAbiParameters, parseAbiParameters, } from 'viem';
import { getChainId, signTypedData } from 'viem/actions';
import { abstractTestnet } from 'viem/chains';
import { assertCurrentChain, getAction, parseAccount } from 'viem/utils';
import {} from 'viem/zksync';
import { VALIDATOR_ADDRESS } from '../constants.js';
import { assertEip712Request, } from '../eip712.js';
import { AccountNotFoundError } from '../errors/account.js';
import { transformHexValues } from '../utils.js';
const ALLOWED_CHAINS = [abstractTestnet.id];
export async function signTransaction(client, signerClient, args, useSignerAddress = false) {
    const { account: account_ = client.account, chain = client.chain, ...transaction } = args;
    // TODO: open up typing to allow for eip712 transactions
    transaction.type = 'eip712';
    transformHexValues(transaction, [
        'value',
        'nonce',
        'maxPriorityFeePerGas',
        'gas',
        'value',
        'chainId',
        'gasPerPubdata',
    ]);
    if (!account_)
        throw new AccountNotFoundError({
            docsPath: '/docs/actions/wallet/signTransaction',
        });
    const smartAccount = parseAccount(account_);
    const fromAccount = useSignerAddress ? signerClient.account : smartAccount;
    assertEip712Request({
        account: smartAccount,
        chain,
        ...transaction,
    });
    if (!chain || !ALLOWED_CHAINS.includes(chain.id)) {
        throw new BaseError('Invalid chain specified');
    }
    if (!chain?.custom?.getEip712Domain)
        throw new BaseError('`getEip712Domain` not found on chain.');
    if (!chain?.serializers?.transaction)
        throw new BaseError('transaction serializer not found on chain.');
    const chainId = await getAction(client, getChainId, 'getChainId')({});
    if (chain !== null)
        assertCurrentChain({
            currentChainId: chainId,
            chain: chain,
        });
    const eip712Domain = chain?.custom.getEip712Domain({
        ...transaction,
        chainId,
        from: fromAccount.address,
        type: 'eip712',
    });
    const rawSignature = await signTypedData(signerClient, {
        ...eip712Domain,
        account: signerClient.account,
    });
    let signature;
    if (useSignerAddress) {
        signature = rawSignature;
    }
    else {
        // Match the expect signature format of the AGW smart account
        signature = encodeAbiParameters(parseAbiParameters(['bytes', 'address', 'bytes[]']), [rawSignature, VALIDATOR_ADDRESS, []]);
    }
    return chain?.serializers?.transaction({
        chainId,
        ...transaction,
        from: fromAccount.address,
        customSignature: signature,
        type: 'eip712',
    }, { r: '0x0', s: '0x0', v: 0n });
}
//# sourceMappingURL=signTransaction.js.map